remove_common_nested_columns: Remove non-list columns when same are present in a list column
Description
Remove non-list columns when same are present in a list column
Usage
remove_common_nested_columns(df, list_column)
Arguments
- df
input dataframe
- list_column
Name or expr of the column which is a list of named lists